@@ -535,7 +535,7 @@ static void delete_files(struct file_list *flist)
   if (!(local_file_list = send_file_list(-1,1,&dot)))
     return;
 
-  for (i=local_file_list->count;i>=0;i--) {
+  for (i=local_file_list->count-1;i>=0;i--) {
     if (!local_file_list->files[i].name) continue;
     if (-1 == flist_find(flist,&local_file_list->files[i])) {
       delete_one(&local_file_list->files[i]);
